/*global define, describe, beforeEach, afterEach, it, expect, sinon, jasmine, spyOn, $, window */
/*jslint todo: true*/
define([
    'app/views/editor',
    'app/models/page'
], function (EditorView, PageModel) {

    'use strict';

    describe('View :: Editor', function () {

        beforeEach(function () {
            this.app = {
                models: {
                    user: {
                        get: sinon.stub()
                    }
                }
            };
            // initialize an editorview
            this.view = new EditorView({
                app: this.app
            });
            // initialize a pagemodel
            this.model = new PageModel({
                id: '1',
                rows: [{
                    id: '1'
                }]
            });
            // set pagemodels url
            this.model.url = function () { return '/rs/page/1'; };
        });

        afterEach(function () {
            //must remove backbone view to avoid side-effects with other tests
            this.view.close();
        });

        it('.initialize() should create a div', function () {
            expect(this.view.el.nodeName).toEqual('DIV');
        });

        it('Rendering should render the view', function () {
            this.view.render();

            expect(this.view.$el).toContain('#tools');
            expect(this.view.$el).toContain('.content');
        });

        it('.updateSiteCanvasClass() should update the class', function () {
            this.view.render();

            expect(this.view.$el).toHaveClass('site-preview');
            this.view.updateSiteCanvasClass('large');
            expect(this.view.$el).toHaveClass('site-large');
        });

        it('.openPage() should cache the page model', function () {
            this.view.render();
            this.view.openPage(this.model);
            this.view.ui.iframe.trigger('load');

            expect(this.view.pageModel).toEqual(this.model);
            expect(this.view.ui.iframe.attr('src')).toEqual(this.model.url());
        });

        describe('.emptyPage()', function () {
            it('should reset the iframe', function () {
                this.view.render();
                this.view.openPage(this.model);
                this.view.closePage(this.model);

                expect(this.view.pageModel).toBeNull();
                expect(this.view.ui.iframe.attr('src')).toEqual('');
            });

            it('should not reset the iframe', function () {
                this.view.render();
                this.view.openPage(this.model);
                this.view.closePage({});

                expect(this.view.pageModel).toBe(this.model);
                expect(this.view.ui.iframe.attr('src')).toEqual(this.model.url());
            });
        });

        describe('savePage:', function () {
            var saveResponse = {
                done : function (callback) { callback(); }
            };
            beforeEach(function () {
                this.view.pageModel = this.model;
                spyOn(this.model, 'save').andReturn(saveResponse);
                spyOn(saveResponse, 'done').andCallThrough();
                spyOn(this.view, 'openPage');
                this.view.savePage();
            });

            it('must save the pageModel', function () {
                expect(this.model.save).toHaveBeenCalled();
            });

            it('must reopen the page with the new page model', function () {
                expect(this.view.openPage).toHaveBeenCalledWith(this.model);
            });
        });

        it('.getLastDropped must return a jQuery element', function () {
            this.view.mouse = {getLastDropped : function () {return 'someElement'; }};

            expect(this.view.getLastDropped()).toEqual(jasmine.any($));
        });

        it('.getLastHovered must return a jQuery element', function () {
            this.view.mouse = {lastHovered: '<div></div>'};

            expect(this.view.getLastHovered()).toEqual(jasmine.any($));
        });

        describe('sendContent:', function () {
            var mockContent = {},
                mockElement = {
                    empty : function () { return; },
                    append : function () {return; },
                    is : function () {return; },
                    data: function () {return 1; }
                };

            beforeEach(function () {
                this.view.pageModel = this.model;
                spyOn(this.model, 'fetch');
                spyOn(this.model, 'addContent');
                spyOn(this.view, 'savePage');
                spyOn(mockElement, 'empty').andReturn(mockElement);
                spyOn(mockElement, 'append');

                this.server = sinon.fakeServer.create();
                this.server.respondWith('POST', '/page/1/column/1/content', [200, {
                    'Content-Type': 'text/html'
                }, 'test content']);
            });

            afterEach(function () {
                this.server.restore();
            });

            it('must append the content rendered by the server to the target', function () {
                spyOn(mockElement, 'is').andReturn(true);

                this.view.sendContent(mockElement, mockContent);
                this.server.respond();

                expect(mockElement.empty.calls.length).toBe(1);
                expect(mockElement.append).toHaveBeenCalledWith('test content');
                expect(this.model.fetch.calls.length).toBe(1);
            });

            it('must add the content to the page and save the page', function () {
                spyOn(mockElement, 'is').andReturn(false);

                this.view.sendContent(mockElement, mockContent);
                this.server.respond();

                expect(this.model.addContent).toHaveBeenCalledWith(mockContent);
                expect(this.view.savePage.calls.length).toBe(1);
            });

        });

        describe('removeContent:', function () {

            var mockElement = {
                empty: sinon.spy(),
                data: sinon.stub().returns(1)
            };
            beforeEach(function () {
                this.view.pageModel = this.model;

                this.mockContent = {};
                this.view.getLastHovered = function () {
                    return {
                        parent: function () {
                            return mockElement;
                        }
                    };
                };
                this.fetchSpy = sinon.spy(this.model, 'fetch');
                this.confirmStub = sinon.stub(window, 'confirm').returns(true);

                this.server = sinon.fakeServer.create();
                this.server.respondWith('DELETE', '/page/1/column/1/content', [204, {
                    'Content-Type': 'application/json'
                }, '']);
            });

            afterEach(function () {
                mockElement.empty.reset();
                this.server.restore();
                this.fetchSpy.restore();
                this.confirmStub.restore();
            });

            it('must delete the content and remove it from DOM', function () {
                this.app.models.user.get.returns(false);
                this.view.removeContent();
                this.server.respond();

                expect(mockElement.empty).toHaveBeenCalledOnce();
                expect(this.fetchSpy).toHaveBeenCalledOnce();
            });

            it('must delete the content and remove it from DOM', function () {
                this.app.models.user.get.returns(true);
                this.view.removeContent();
                this.server.respond();

                expect(mockElement.empty).toHaveBeenCalledOnce();
                expect(this.fetchSpy).toHaveBeenCalledOnce();
                expect(this.confirmStub).toHaveBeenCalled();
            });

            it('must *not* delete the content and remove it from DOM', function () {
                this.app.models.user.get.returns(true);
                this.confirmStub.returns(false);
                this.view.removeContent();
                this.server.respond();

                expect(mockElement.empty).not.toHaveBeenCalled();
                expect(this.fetchSpy).not.toHaveBeenCalled();
                expect(this.confirmStub).toHaveBeenCalled();
            });

        });

        describe('content functions:', function () {
            var data = {};

            beforeEach(function () {
                this.sendContentSpy = spyOn(this.view, 'sendContent');
            });

            it('change content should call sendContent with the lastDropped target', function () {
                var lastHovered = {parent: function () {return; }};
                this.lastHoveredSpy = spyOn(this.view, 'getLastHovered').andReturn(lastHovered);
                this.view.changeContent(data);

                expect(this.lastHoveredSpy).toHaveBeenCalled();
                expect(this.sendContentSpy).toHaveBeenCalledWith(lastHovered.parent(), data);
            });

            it('add content should call sendContent with the lastDropped target', function () {
                var lastDropped = {};
                this.lastDroppedSpy = spyOn(this.view, 'getLastDropped').andReturn(lastDropped);

                this.view.addContent(data);

                expect(this.lastDroppedSpy).toHaveBeenCalled();
                expect(this.sendContentSpy).toHaveBeenCalledWith(lastDropped, data);
            });

        });

        describe('addRow:', function () {

            beforeEach(function () {
                this.view.pageModel = this.model;
                this.saveSpy = spyOn(this.view, 'savePage');
            });

            it('must add a row to the page and save it', function () {
                expect(this.view.pageModel.get('rows').length).toBe(1);

                this.view.addRow({});

                expect(this.view.pageModel.get('rows').length).toBe(2);
                expect(this.saveSpy).toHaveBeenCalled();
            });

        });
    });
});
