/*global define, describe, beforeEach, spyOn, it, expect, $, afterEach*/
define([
    'app/views/mouse', 'sandbox/event'
], function (Mouse, vent) {

    'use strict';

    describe('View :: Mouse', function () {

        beforeEach(function () {

            // Init dummy domNode to attach the view to.
            this.$el = $('<div></div>');
            this.$dataEl = $('<div data-type=""></div>');

            // Init `SUT`.
            this.view = new Mouse({
                el: this.$el.get(0)
            });

            // Reset `lastHovered` for testing.
            this.setLastHovered = function (node) {
                this.view.lastHovered = node;
            };

            // Init `spy` on `trigger.
            spyOn(vent, 'trigger');

        });

        afterEach(function () {

            // Clean up.
            this.view.remove();

        });

        describe('On Enter', function () {

            beforeEach(function () {

                this.$el.trigger('mouseenter');

            });

            it('should trigger a null >hovered.last< event', function () {

                expect(vent.trigger).toHaveBeenCalledWith(this.view.lastHoveredEvent, null);

            });

        });

        describe('On Move', function () {

            it('should trigger a null >hovered.last< event if the target is not yet hovered', function () {

                this.$el.trigger('mousemove');

                expect(vent.trigger).toHaveBeenCalledWith('outline.actions:hide');
                expect(vent.trigger).toHaveBeenCalledWith(this.view.lastHoveredEvent, this.$el.get(0), undefined);

            });

            it('should trigger a >outline.actions:show< event', function () {
                this.view.hover(this.$dataEl.get('0'));

                expect(vent.trigger).toHaveBeenCalledWith('outline.actions:show');
            });

            it('shouldn\'t trigger any event if the target is already hovered', function () {

                this.setLastHovered(this.$el.get(0));

                this.$el.trigger('mousemove');

                expect(vent.trigger).not.toHaveBeenCalled();

            });


        });

        describe('On Leave', function () {

            beforeEach(function () {
                this.$el.trigger('mouseleave');
            });

            it('should trigger a null >hovered.last< event', function () {
                expect(vent.trigger).toHaveBeenCalledWith(this.view.lastHoveredEvent, null);
            });

            it('should clear the lastHovered element', function () {
                expect(this.view.lastHovered).toBeNull();
            });

        });

        describe('On Drag Leave', function () {

            beforeEach(function () {
                this.$el.trigger('dragleave');
            });

            it('should trigger a null >hovered.last< event', function () {
                expect(vent.trigger).toHaveBeenCalledWith(this.view.lastHoveredEvent, null);
            });

            it('should clear the lastHovered element', function () {
                expect(this.view.lastHovered).toBeNull();
            });

        });

        describe('On Drop', function () {

            it('should trigger a >target:drop< event', function () {

                this.$el.trigger('drop');

                expect(vent.trigger).toHaveBeenCalledWith(this.view.dropTargetEvent, this.$el.get(0));

            });

            it('should cache the target', function () {

                this.$el.trigger('drop');

                expect(this.view.getLastDropped()).toBe(this.$el.get(0));

            });
        });

        describe('On DragOver', function () {

            beforeEach(function () {

                this.mockEvent = $.Event('dragover', {
                    originalEvent : {
                        dataTransfer : {
                            types : ['div[data-view="column"]']
                        }
                    }
                });

                spyOn(this.mockEvent, 'preventDefault');

            });

            it('should prevent the event defaults if target is dropzone', function () {
                this.mockEvent.target = $('<div class="col-medium-6" data-view="column">');
                this.$el.triggerHandler(this.mockEvent);
                expect(this.mockEvent.preventDefault).toHaveBeenCalled();
            });

            it('should not prevent the event defaults if target is not a dropzone', function () {
                this.mockEvent.target = $('<div class="col-medium-6" data-view="row">');
                this.$el.triggerHandler(this.mockEvent);
                expect(this.mockEvent.preventDefault).not.toHaveBeenCalled();
            });

        });

    });
});
